#include "testlib.h"
#include <cassert>
#include <map>
#include <numeric>
#include <set>
#include <string>
#define ld long double

using namespace std;

const int RND = 1e9;

ld gen_double() {
	return (ld)rnd.next(0, RND) / (ld)RND;
}

map<string, string> params;
set<string> options;

void parse_params(int argc, char* argv[]) {
	for (int i = 1; i < argc; ++i) {
		string param(argv[i]);
		auto pos = param.find('=');
		if (pos == string::npos) {
			options.insert(param);
		} else {
			params[param.substr(0, pos)] = param.substr(pos + 1);
		}
	}
}

vector<string> splitStr(const string& s, char delimiter) {
	vector<string> res = {""};
	for (auto c : s) {
		if (c == delimiter) {
			res.push_back("");
		} else {
			res.back().push_back(c);
		}
	}
	return res;
}

vector<string> parseArray(const string& s) {
	return splitStr(s.substr(1, (int)s.size() - 2), ',');
}

int n, C_min, C_max;
vector<int> arr;

void print() {
	cout << arr.size() << '\n';
	for (int i = 0; i < (int)arr.size(); ++i) {
		if (i) {
			cout << ' ';
		}
		cout << arr[i];
	}
	cout << '\n';
}

void fetchC() {
	if (params.count("C")) {
		C_min = 1;
		C_max = stoi(params["C"]);
	} else {
		C_min = stoi(params["C_min"]);
		C_max = stoi(params["C_max"]);
	}
}

vector<int> perm, values;

void gen_values_randomly() {
	values.resize(n);
	for (int i = 0; i < n; ++i) {
		values[i] = rnd.next(C_min, C_max);
	}
}

void gen_values_from_cnt() {
	assert(C_min == 1);
	vector<string> _cnt = parseArray(params["cnt"]);
	for (int i = 0; i < C_max; ++i) {
		int occ = stoi(_cnt[i]);
		for (int j = 0; j < occ; ++j) {
			values.push_back(i + 1);
		}
	}
	assert(values.size() == n);
}

void gen_values_from_distribution() {
	assert(C_min == 1);
	vector<string> _distr = parseArray(params["distr"]);
	vector<ld> distr(C_max);
	transform(_distr.begin(), _distr.end(), distr.begin(),
		[] (string a) { return stold(a); });
	assert(abs(accumulate(distr.begin(), distr.end(), (ld)0.0) - 1.0) < 1e-9);
	for (int i = 0; i < n; ++i) {
		ld val = gen_double();
		for (int j = 0; j < C_max; ++j) {
			if (val < distr[j]) {
				values.push_back(j + 1);
				break;
			} else {
				val -= distr[j];
			}
		}
	}
}

void gen_distinct_values() {
	int len = C_max - C_min + 1;
	assert(len >= n);
	if (len <= 2 * n) {
		vector<int> _values(len);
		iota(_values.begin(), _values.end(), C_min);
		shuffle(_values.begin(), _values.end());
		values = vector<int>(_values.begin(), _values.begin() + n);
	} else {
		set<int> _values;
		for (int i = 0; i < n; ++i) {
			while (true) {
				int val = rnd.next(C_min, C_max);
				if (!_values.count(val)) {
					_values.insert(val);
					break;
				}
			}
		}
		values = vector<int>(_values.begin(), _values.end());
	}
}

void gen_random_permutation() {
	perm.resize(n);
	iota(perm.begin(), perm.end(), 0);
	shuffle(perm.begin(), perm.end());
}

int skL, skR, ptr;
ld frac, rec;

void skew(int l, int r) {
	if (l > r) {
		return;
	}
	int mid = gen_double() < frac 
		? max(l, r - rnd.next(0, skR - 1))
		: min(r, l + rnd.next(0, skL - 1));
	perm[mid] = ptr--;
	if (gen_double() < rec) {
		skew(mid + 1, r);
		skew(l, mid - 1);
	} else {
		skew(l, mid - 1);
		skew(mid + 1, r);
	}
}

void gen_skewed_permutation() {
	perm.resize(n);
	skL = params.count("skL") ? stoi(params["skL"]) : 1;
	skR = params.count("skR") ? stoi(params["skR"]) : 1;
	frac = params.count("frac") ? stold(params["frac"]) : 0.5;
	rec = params.count("rec") ? stold(params["rec"]) : 0;
	ptr = n - 1;
	skew(0, n - 1);
}

void combine() {
	sort(values.begin(), values.end());
	arr.resize(n);
	for (int i = 0; i < n; ++i) {
		arr[i] = values[perm[i]];
	}
}

int main(int argc, char* argv[]) {
	registerGen(argc, argv, 1);
	parse_params(argc, argv);
	if (params.count("n")) {
		n = stoi(params["n"]);
	} else {
		n = rnd.next(stoi(params["n_min"]), stoi(params["n_max"]));
	}
	fetchC();
	if (params.count("cnt")) {
		gen_values_from_cnt();
	} else if (params.count("distr")) {
		gen_values_from_distribution();
	} else if (options.count("distinct")) {
		gen_distinct_values();
	} else {
		gen_values_randomly();
	}
	if (options.count("skewed")) {
		gen_skewed_permutation();
	} else {
		gen_random_permutation();
	}
	combine();
	print();
}
